/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich  
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Commando Interpreter

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "cmd.h"

volatile unsigned char variable[MAX_VAR];	
volatile unsigned int frame[MAX_VAR];		// MAX_VAR 
volatile unsigned int status_alt[9];		// gespeicherter Zustand


//------------------------------------------------------------------------------
// Kommando auswerten
unsigned char extract_cmd(char frame[10])
{
	//Stringzeiger;
	unsigned int befehl; // Kommando
	unsigned int parameter; // bergebene Parameter
	char *frame_tmp; // temp. Frame

	//Variablen finden und auswerten
	for (unsigned char a = 0; a<MAX_VAR;a++)
	{ 
		frame_tmp = strsep(&frame," ");			// String zerlegen
		variable[a] = strtol(frame_tmp,NULL,16);// Werte zuweisen
	}

	if (variable[0] != 02)						// Startzeichen nicht STX (0x02)
	{
		return(0);								// Rcksprung --> NAK senden
	}

	befehl = variable[1]; 						// Befehl bernehmen
	parameter = variable[2];					// Parameter fr Befehl bernehmen

	if (variable[3] != 03) 						// Endezeichen nicht ETX (03)
	{
		return(0);								// Rcksprung --> NAK senden
	}

	// Befehl bersetzen und ausfhren
    switch(befehl)
	{
	case 16:	// 10 - Zustand aller digitalen Eingnge lesen
		command_getdinbin();
		break;
	case 17:	// 11 - Zustand eines digitalen Eingangs lesen
		command_getdin(parameter);
		break;
	case 33:	// 21 - Zustand aller digitalen Ausgnge lesen
		command_getoutbin();	
		break;
	case 34:	// 22 - Zustand eines digitalen Ausgangs lesen
		command_getout(parameter);
		break;
	case 37:	// 25 - Zustand digitale Ausgnge setzen
		command_setout(parameter);
		break;
	case 38: 	// 26 - Zustand digitale Ausgnge zurcksetzen
		command_resetout(parameter);
		break;
	case 39:	// 27 - Zustand digitale Ausgnge setzen
		command_setoutbin(parameter);
		break;
	case 40: 	// 28 - Zustand digitale Ausgnge zurcksetzen
		command_resetoutbin(parameter);
		break;
	case 48:	// 30 - Zustand eines analogen Eingangs lesen
		command_getain (parameter);
		break;
	}
	return(1); 									// Rcksprung --> ACK senden
}

//------------------------------------------------------------------------------
//--> Zustand der digitalen Eingnge abfragen
void command_getdinbin (void)
{
	usart_write(STX "10 %2x" ETX,PINA&0b00001111); // Meldung senden
}

//------------------------------------------------------------------------------
//--> nderung des Zustands eines einzelnen digitalen Eingangs abfragen
void command_getdinx (int inp)
{
	uint8_t status;							// Variable fr aktuellen Statuswert
	uint8_t inp1; 						    // Variable fr die gewandelte Eingang-Nr
	inp1 = 1 <<inp;							// Eingang 0,1,2,3 umwandeln in 1,2,4,8
	if ((PINA&inp1) == inp1)				// Bit[inp] gesetzt --> Eingang offen
		status = 00;						
	else
		status = 01;						// Bit[inp] nicht gesetzt --> Eingang geschlossen
	if (status != status_alt[inp])			// Status seit letzter Abfrage gendert?
	{
		usart_write(STX "11 %2i %2i" ETX,inp,status); // Meldung senden
		status_alt[inp] = status;			// Status in Array abspeichern
	}
}

//------------------------------------------------------------------------------
//--> Zustand eines einzelnen digitalen Eingangs abfragen
void command_getdin (int inp)
{
	uint8_t status;							// Variable fr aktuellen Statuswert
	uint8_t inp1; 						    // Variable fr die gewandelte Eingang-Nr
	inp1 = 1 <<inp;							// Eingang 0,1,2,3 umwandeln in 1,2,4,8
	if ((PINA&inp1) == inp1)				// Bit[inp] gesetzt --> Eingang offen
		status = 00;						
	else
		status = 01;						// Bit[inp] nicht gesetzt --> Eingang geschlossen
	
	usart_write(STX "11 %2x %2i" ETX, inp, status); // Meldung senden
}

//------------------------------------------------------------------------------
//--> Zustand aller digitalen Ausgnge lesen
void command_getoutbin(void)
{
	usart_write(STX "21 %2x" ETX, PINC&0b11111111); // Meldung senden
}

//------------------------------------------------------------------------------
//--> Zustand eines digitalen Ausgangs lesen
void command_getout(int outp)
{
	uint8_t status;
	uint8_t outp1;						// Variable fr gewandelte Ausgang-Nr
	outp1 = 1<<outp;					// Ausgang 0,1,2,3,4,5,6,7 in 1,2,4,8,16,... wandeln
	if ((PINC&outp1) == outp1)			// Bit[inp] gesetzt --> Eingang offen
		status = 01;						
	else
		status = 00;

	usart_write(STX "22 %2i %2i" ETX, outp, status); // Meldung senden
}

//------------------------------------------------------------------------------
//--> Mewert eines analogen Eingangs abfragen, bei nderung weitergeben
void command_getainx (int inp)
{
	uint16_t analog;							// Variable fr aktuellen Messwert
	uint16_t diff;								// Differenz aktueller / gespeicherter Mewert
	analog = (var_array[inp]);					// Messwert fr Eingang "inp" bergeben
	if (analog > status_alt[inp])				// Gemessener Wert grer gespeicherter Wert ?
		diff = analog - status_alt[inp];		// Delta ermitteln
	else
		diff = status_alt[inp] - analog;		// Delta ermittlen
	
	// Messwert bernehmen
	if (diff >= delta[0]) 				// Unterschied seit letzter Messung >= Delta
	{
		usart_write(STX "30 %2i %2i" ETX, inp, analog); // Meldung senden
		status_alt[inp] = analog;			// aktuellen Messwert in Array abspeichern
	}
}

//------------------------------------------------------------------------------
//--> Mewert eines analogen Eingangs abfragen
void command_getain (int inp)
{
	uint16_t analog;							// Variable fr aktuellen Messwert
	analog = (var_array[inp]);					// Messwert bernehmen

	usart_write(STX "30 %2i %2i" ETX, inp, analog); // Meldung senden
} 

//------------------------------------------------------------------------------
//--> Zustand der digitalen Ausgnge setzen
void command_setout(int out)
{
	uint8_t out1; 
	out1 = 1<<out;
	PORTC = PORTC | out1; // Byte setzen
}

//------------------------------------------------------------------------------
//--> Zustand eines Ports zurcksetzen
void command_resetout (int out)
{
	uint8_t out1;
	out1 = 1<<out;
	PORTC = PORTC & ~ out1; // Byte zurcksetzen
}

//------------------------------------------------------------------------------
//--> Zustand aller digitalen Ausgnge setzen
void command_setoutbin(int out)
{
	PORTC = PORTC | out; // Byte setzen
}

//------------------------------------------------------------------------------
//--> Zustand aller Ports zurcksetzen
void command_resetoutbin(int out)
{
	PORTC = PORTC & ~ out; // Byte zurcksetzen
}

//------------------------------------------------------------------------------
// Reset ausfhren
void command_reset (void)
{
	RESET();
}

//------------------------------------------------------------------------------
// Wake on LAN Meldung verschicken
void command_wol (void)
{
	#if USE_WOL
	// EEPROM beschreiben, falls Parameter angegeben wurden
	if ((*((unsigned int*)&variable[0]) != 0x00000000) || (*((unsigned int*)&variable[1]) != 0x00000000) || (*((unsigned int*)&variable[2]) != 0x00000000))
	{	
		//schreiben der MAC
		for (unsigned char count = 0; count<6; count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(WOL_MAC_EEPROM_STORE + count),variable[count]);
		}
		//zustzlich schreiben der Broadcast-Adresse, falls vorhandenden
		for (unsigned char count = 0; count<4 && (*((unsigned int*)&variable[6]) != 0x00000000);count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char*)(WOL_BCAST_EEPROM_STORE + count),variable[count+6]);
		}
		//init
		wol_init();
	}else{
		//MagicPacket senden
		wol_request();
	}
	#endif //USE_WOL	
}

//------------------------------------------------------------------------------
//save all ip-datas
void save_ip_addresses(void)
{
	// genderte MAC-Adresse abspeichern
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC1_EEPROM_STORE),mymac[0]);
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC2_EEPROM_STORE),mymac[1]);
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC3_EEPROM_STORE),mymac[2]);
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC4_EEPROM_STORE),mymac[3]);
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC5_EEPROM_STORE),mymac[4]);
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(MYMAC6_EEPROM_STORE),mymac[5]);
  	// genderte IP-Adresse abspeichern
  	for (unsigned char count = 0; count<4; count++)
  	{
   	 	eeprom_busy_wait ();
		eeprom_write_byte((unsigned char *)(IP_EEPROM_STORE + count),myip[count]);
  	}
  	// genderte Netmask abspeichern
  	for (unsigned char count = 0; count<4; count++)
  	{
    	eeprom_busy_wait ();
    	eeprom_write_byte((unsigned char *)(NETMASK_EEPROM_STORE + count),netmask[count]);
 	}
  	// genderte Gateway IP-Adresse abspeichern
  	for (unsigned char count = 0; count<4; count++)
  	{
    	eeprom_busy_wait ();
    	eeprom_write_byte((unsigned char *)(ROUTER_IP_EEPROM_STORE + count),router_ip[count]);
 	}
  	// genderten Delta-Wert fr analoge Eingnge abspeichern
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(DELTA_EEPROM_STORE),delta[0]);
  	// genderte CMD-TELNET Einstellung abspeichern
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(CMD_TELNET_EEPROM_STORE),ctel);
  	// genderter Username abspeichern
 // eeprom_busy_wait ();
 // eeprom_write_byte((unsigned char *)(USER_EEPROM_STORE),user[0]);
 // gendertes Passwort abspeichern
 // eeprom_busy_wait ();
 // eeprom_write_byte((unsigned char *)(PASS_EEPROM_STORE),pass[0]);
 	// genderte DNS-Server-Adresse abspeichern
  	#if USE_DNS
  	for (unsigned char count = 0; count<4; count++)
  	{
    	eeprom_busy_wait ();
    	eeprom_write_byte((unsigned char *)(DNS_IP_EEPROM_STORE + count),dns_server_ip[count]);
  	}
  	#endif //USE_DNS
}

// Grundeinstellungen (siehe config.h) wieder herstellen
void modul_init(void)
{
  	(*((unsigned long*)&mymac[0])) = MYMAC1;
  	(*((unsigned long*)&mymac[1])) = MYMAC2;
	(*((unsigned long*)&mymac[2])) = MYMAC3;
	(*((unsigned long*)&mymac[3])) = MYMAC4;
	(*((unsigned long*)&mymac[4])) = MYMAC5;
	(*((unsigned long*)&mymac[5])) = MYMAC6;
	(*((unsigned long*)&myip[0])) = MYIP;
	(*((unsigned long*)&netmask[0])) = NETMASK;
	(*((unsigned long*)&router_ip[0])) = ROUTER_IP;
	(*((unsigned long*)&delta[0])) = DELTA;
	(*((unsigned long*)&ctel)) = CMD_TELNET;
	save_ip_addresses();	// Werte in's EEPROM schreiben
//	RESET(); 		// Modul-Reset
}










