/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Commando Interpreter

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CMD_H_
	#define _CMD_H_	
		
	#include <avr/io.h>
	#include <string.h>
	#include <stdlib.h>
	#include <avr/eeprom.h>
	#include "usart.h"
	#include "stack.h"
	#include "httpd.h"
	#include "wol.h"
 	
	typedef struct
	{
		char* cmd; 			// Stringzeiger auf Befehlsnamen
		void(*fp)(void);  	// Zeiger auf auszufhrende Funktion
	} COMMAND_STRUCTUR;	
	
	#define MAX_VAR	10		
	#define STX "\x02" 		// Konstante fr Startzeichen
	#define ETX "\x03" 		// Konstante fr Endezeichen
	#define ACK "\x06" 		// Konstante fr positive Besttigung
	#define NAK "\x15"		// Konstante fr negative Besttigung
	
	extern unsigned char extract_cmd (char frame[10]);	// Kommandos erkennen / ausfhren
	extern void write_eeprom_ip (unsigned int);		// Einzelnen Wert in EEPROM speichern
    extern void save_ip_addresses(void);	// Genderte Werte im EEPROM speichern
   
	// Modul zurcksetzen
	extern void modul_init			(void); // Modul auf Grundeinstellungen (siehe config.h) zurcksetzen
	#define RESET() {asm("ldi r30,0"); asm("ldi r31,0"); asm("ijmp");} // Modulreset
	
	// zustzliche Befehle zum auslesen / steuern der digitalen Eingnge
	extern void command_getdin		(int inp); 	// Zustand eines dig. Eingangs lesen
	extern void command_getdinbin	(void); 	// Zustand aller dig. Eingnge lesen
	extern void command_getdinx		(int inp); 	// Zustandsnderung eines dig. Eing. abfragen

	// zustzliche Befehle zum auslesen / steuern der analogen Eingnge
	extern void command_getain 		(int inp); 	// Analoge Eingnge abfragen
	extern void command_getainx		(int inp); 	// Zustandsnderung eines anal. Eing. abfragen 
	extern void command_getdelta	(void);		// Delta aktueller / gespeicherter anal. Messwert abfragen

	// zustzliche Befehle zum auslesen / steuern der digitalen Ausgnge
	extern void command_setout		(int outp); // Einzelnen Ausgang setzen 00... 07
	extern void command_setoutbin	(int outp); // Mehrere Ausgnge setzen 00...255
	extern void command_resetout	(int outp); // Einzelnen Ausgang setzen 00... 07
	extern void command_resetoutbin	(int outp); // Mehrere Ausgnge zurcksetzen 00...255
	extern void command_getout		(int outp); // Zustand eines dig. Ausgangs lesen 00...07
	extern void command_getoutbin	(void); 	// Zustand aller dig. Ausgnge lesen 00...255

	#if USE_WOL									// Wake on LAN
	extern void command_wol 		(void);
	#endif //USE_WOL
	
#endif //_CMD_H_


