/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich  -  angepasst an AVR-NET-IO durch ** EHB **
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver Applikation

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "httpd.h"
#include "webpage.h"
#include "lcd.h"
#include "stack.h"		// hinzugefgt um auf Werte wie myip[] ... zugreifen zu knnen

struct http_table http_entry[MAX_TCP_ENTRY];

//Hier wird das codierte Passwort aus config.h gespeichert.
unsigned char http_auth_passwort[30];
unsigned char post_in[5] = {'O','U','T','='}; // Start "POST" - Daten
unsigned char post_ready[5] = {'S','U','B','='}; // Ende "POST" - 
unsigned char PORT_tmp = 0;	
char htm_seite[10];					
char dstr[24]={"No Time...             "};
unsigned int input;
long int iwert;

//----------------------------------------------------------------------------
//Variablenarry zum einfgen in Webseite %VA@00 bis %VA@09
unsigned int var_array[MAX_VAR_ARRAY] = {10,50,30,0,0,0,0,0,0,0};
//----------------------------------------------------------------------------

PROGMEM char http_header1[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR_Small_Webserver\r\n"
								"Content-Type: text/html\r\n\r\n"};

PROGMEM char http_header2[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR_Small_Webserver\r\n"
								"Content-Type: image/jpg\r\n\r\n"};

PROGMEM char http_header3[]={	"HTTP/1.0 401 Unauthorized\r\n"
								"Server: AVR_Small_Webserver\r\n"
								"WWW-Authenticate: Basic realm=\"NeedPassword\""
								"\r\nContent-Type: text/html\r\n\r\n"};

//----------------------------------------------------------------------------
//Kein Zugriff Seite bei keinem Passwort
PROGMEM char Page0[] = {"401 Unauthorized%END"};

unsigned char rx_header_end[5] = {"\r\n\r\n\0"};

//----------------------------------------------------------------------------
//Initialisierung des Httpd Testservers
void httpd_init (void)
{
	//HTTP_AUTH_STRING 
	decode_base64((unsigned char*)HTTP_AUTH_STRING,http_auth_passwort);

	//Serverport und Anwendung eintragen
	add_tcp_app (HTTPD_PORT, (void(*)(unsigned char))httpd);
}
   
//----------------------------------------------------------------------------
//http Testserver
void httpd (unsigned char index)
{
    //Verbindung wurde abgebaut!
    if (tcp_entry[index].status & FIN_FLAG)
    {
        return;
    }

	//Allererste Aufruf des Ports fr diese Anwendung
	//HTTPD_Anwendungsstack lschen
	if(tcp_entry[index].app_status==1)
	{
		httpd_stack_clear(index);
	}
	
	//HTTP wurde bei dieser Verbindung zum ersten mal aufgerufen oder
	//HTML Header Retransmission!
	if (tcp_entry[index].app_status <= 2)
	{	
		httpd_header_check (index);
		return;
	}
	
	//Der Header wurde gesendet und mit ACK besttigt (tcp_entry[index].app_status+1)
	//war das HTML Packet fertig, oder mssen weitere Daten gesendet werden, oder Retransmission?
	if (tcp_entry[index].app_status > 2 && tcp_entry[index].app_status < 0xFFFE)
	{
		httpd_data_send (index);
		return;
	}
	
	//Verbindung kann geschlossen werden! Alle HTML Daten wurden gesendet TCP Port kann
	//geschlossen werden (tcp_entry[index].app_status >= 0xFFFE)!!
	if (tcp_entry[index].app_status >= 0xFFFE)
	{
		tcp_entry[index].app_status = 0xFFFE;
		tcp_Port_close(index);
		return;
	}
	return;
}

//----------------------------------------------------------------------------
//HTTPD_STACK lschen
void httpd_stack_clear (unsigned char index)
{
	http_entry[index].http_header_type =0;
	http_entry[index].first_switch = 0;
	http_entry[index].http_auth = HTTP_AUTH_DEFAULT;
	http_entry[index].new_page_pointer = 0;
	http_entry[index].old_page_pointer = 0;
	http_entry[index].post = 0;
	http_entry[index].auth_ptr = http_auth_passwort;
	http_entry[index].post_ptr = post_in; // "OUT" gefunden
	http_entry[index].post_ready_ptr = post_ready; // "SUB" gefunden
	http_entry[index].hdr_end_pointer = rx_header_end;
	#if USE_CAM
	http_entry[index].cam = 0;
	#endif //USE_CAM				
	HTTP_DEBUG("\r\n**** NEUE HTTP ANFORDERUNG ****\r\n\r\n");	
	return;
}

//----------------------------------------------------------------------------
//Eintreffenden Daten vom Client checken
void httpd_header_check (unsigned char index)
{
	unsigned int a = 0;	// Zhlvariable
	
	if(strcasestr_P((char*)&eth_buffer[TCP_DATA_START_VAR],PSTR("POST"))!=0) // POST vorhanden?
		{
		http_entry[index].post = 1; // POST enthalten!
		}
	
	//finden der Authorization und das Ende im Header auch ber mehrere Packete hinweg!!	
	if(*http_entry[index].hdr_end_pointer != 0) // Ende-Pointer vorhanden
	{		
		for(a=TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{	
			HTTP_DEBUG("%c",eth_buffer[a]); // Debugmeldung
			
			if(!http_entry[index].http_auth) // Autorisierung nicht vorhanden?
			{
				if (eth_buffer[a] != *http_entry[index].auth_ptr++)
				{
					http_entry[index].auth_ptr = http_auth_passwort;
				}
				if(*http_entry[index].auth_ptr == 0) 
				{
					http_entry[index].http_auth = 1;
					HTTP_DEBUG("  <---LOGIN OK!--->\r\n");
				}
			}
			
			if (eth_buffer[a] != *http_entry[index].hdr_end_pointer++)
			{
				http_entry[index].hdr_end_pointer = rx_header_end;
			}
			
			//Das Headerende wird mit (CR+LF+CR+LF) angezeigt!
			if(*http_entry[index].hdr_end_pointer == 0) 
			{
				HTTP_DEBUG("<---HEADER ENDE ERREICHT!--->\r\n"); // Debugmeldung
				break;
			}
		}
	}
	
	//Einzelne Postpacket (z.B. bei firefox)
	if(http_entry[index].http_auth && http_entry[index].post == 1) // Autorisierung + POST OK
	{
		htm_seite[0] = 0; // Variable fr Namen der HTM-Seite
		for(input=0, a = TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{	
			//Schaltanweisung finden!
			if (eth_buffer[a] != *http_entry[index].post_ptr++) // "OUT" gefunden ?
			{
				http_entry[index].post_ptr = post_in; // "OUT" gefunden
			}
			if(*http_entry[index].post_ptr == 0) // "OUT" gefunden ?
			{
				eth_buffer[a+1+strcspn((char*)(&eth_buffer[a+1]),"&")]=0; // "&"-Zeichen entfernen
				if(!input) strcpy(htm_seite,(char*)(&eth_buffer[a+1])); // Beim 1.Durchlauf Seitennamen bernehmen
				//HTTP_DEBUG("Seite :%s \r\n", htm_seite); // Debugmeldung
				HTTP_DEBUG("Durchgang %i :%c%c%c%c%c%c%c%c%c%c\r\n",input,eth_buffer[a+1],eth_buffer[a+2],eth_buffer[a+3],eth_buffer[a+4],eth_buffer[a+5],eth_buffer[a+6],eth_buffer[a+7],eth_buffer[a+8],eth_buffer[a+9],eth_buffer[a+10]);
				if(!strcmp(htm_seite,"index.htm")) // Indexseite gefunden
				{
					switch (eth_buffer[a+1])
				  	{
				    	case ('A'):						// Ausgang 1
					    	PORT_tmp = PORT_tmp + 1;
				      	break;
				
				    	case ('B'):						// Ausgang 2
					  		PORT_tmp = PORT_tmp + 2;
				      	break;
				
				    	case ('C'):						// Ausgang 3
					  		PORT_tmp = PORT_tmp + 4;
				      	break;
					
						case ('D'):						// Ausgang 4
					  		PORT_tmp = PORT_tmp + 8;
				      	break;
					
						case ('E'):						// Ausgang 5
					  		PORT_tmp = PORT_tmp + 16;
				      	break;

						case ('F'):						// Ausgang 6
					  		PORT_tmp = PORT_tmp + 32;
				      	break;

						case ('G'):						// Ausgang 7
					  		PORT_tmp = PORT_tmp + 64;
				      	break;

						case ('H'):						// Ausgang 8
					  		PORT_tmp = PORT_tmp + 128;
				      	break;

						case ('R'):						// Modulreset
					  		RESET();
				      	break;

						case ('O'):						// Grundeinstellungen wiederherstellen
							modul_init();
				      	break;

					  #if USE_WOL
                    	case 'W':
                       		wol_enable = 1;
						break;
                      #endif //USE_WOL
					} // Ende --> switch (eth_buffer[a+1])
				} // Ende --> if(!strcmp(htm_seite,"index.htm"))

				if(!strcmp(htm_seite,"setup.htm")) // Setup-Seite gefunden
				{
					switch (input)
				  	{
				    	case 1:						// MAC-Adresse - Byte 1
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 1:%x\r\n",iwert);
							mymac[0] = iwert;
				      	break;
				
				    	case 2:						// MAC-Adresse - Byte 2
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 2:%x\r\n",iwert);
							mymac[1] = iwert;
				      	break;
				
				    	case 3:						// MAC-Adresse - Byte 3
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 3:%x\r\n",iwert);
							mymac[2] = iwert;
				      	break;

						case 4:						// MAC-Adresse - Byte 4
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 4:%x\r\n",iwert);
							mymac[3] = iwert;
					  	break;

						case 5:						// MAC-Adresse - Byte 5
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 5:%x\r\n",iwert);
							mymac[4] = iwert;
				      	break;

						case 6:						// MAC-Adresse - Byte 6
							iwert = strtol((char*)&eth_buffer[a+1],NULL,16);
					  		HTTP_DEBUG("MAC 6:%x\r\n",iwert);
							mymac[5] = iwert;
				      	break;

						case 7:						// IP-Adresse - Byte 1
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("IP 1:%i\r\n",iwert);
							myip[0] = iwert;
				      	break;

						case 8:						// IP-Adresse - Byte 2
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("IP 2:%i\r\n",iwert);
							myip[1] = iwert;
				      	break;

						case 9:						// IP-Adresse - Byte 3
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("IP 3:%i\r\n",iwert);
							myip[2] = iwert;
				      	break;

						case 10:					// IP-Adresse - Byte 4
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("IP 4:%i\r\n",iwert);
							myip[3] = iwert;
				      	break;

						case 11:					// Netmask - Byte 1	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("NM 1:%i\r\n",iwert);
							netmask[0] = iwert;
				      	break;

						case 12:					// Netmask - Byte 2	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("NM 2:%i\r\n",iwert);
							netmask[1] = iwert;
				      	break;

						case 13:					// Netmask - Byte 3	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("NM 3:%i\r\n",iwert);
							netmask[2] = iwert;
				      	break;

						case 14:					// Netmask - Byte 4
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("NM 4:%i\r\n",iwert);
							netmask[3] = iwert;
				      	break;

						case 15:					// Gateway - Byte 1	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("GW 1:%i\r\n",iwert);
							router_ip[0] = iwert;
				      	break;

						case 16:					// Gateway - Byte 2	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("GW 2:%i\r\n",iwert);
							router_ip[1] = iwert;
				      	break;

						case 17:					// Gateway - Byte 3	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("GW 3:%i\r\n",iwert);
							router_ip[2] = iwert;
				      	break;

						case 18:					// Gateway - Byte 4
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("GW 4:%i\r\n",iwert);
							router_ip[3] = iwert;
				      	break;

						case 19:					// Delta-Wert analoge Eingnge	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("Delta:%i\r\n",iwert);
							delta[0] = iwert;
				      	break;

						case 20:					// CMD-Schnittstelle Telnet oder USART	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("Telnet:%i\r\n",iwert);
							ctel = iwert;
				      	break;

						case 21:					// Username	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("User:%i\r\n",iwert);
							user[0] = iwert;
				      	break;

						case 22:					// Passwort	
							iwert = strtol((char*)&eth_buffer[a+1],NULL,10);
					  		HTTP_DEBUG("Passwort:%i\r\n",iwert);
							pass[0] = iwert;
				      	break;
					} // Ende --> switch (input)
				} // Ende --> if(!strcmp(htm_seite,"setup.htm"))					 
				http_entry[index].post_ptr = post_in; // "OUT" gefunden
			  	input++;
			//Schaltanweisung wurde gefunden
			} // Ende --> if(*http_entry[index].post_ptr == 0)		

			//Submit schliet die Suche ab!
			if (eth_buffer[a] != *http_entry[index].post_ready_ptr++) // "SUB" gefunden ?
			{
				http_entry[index].post_ready_ptr = post_ready; // "SUB" gefunden
			}
			if(*http_entry[index].post_ready_ptr == 0) // Submit gefunden
			{
				http_entry[index].post = 0;
				//HTTP_DEBUG("Seite nach SUB:%s\r\n", htm_seite); // Debugmeldung
				if(!strcmp(htm_seite,"index.htm")) // Indexseite gefunden, genderte Ausgnge schalten
				{
					HTTP_DEBUG("Genderte Ausgnge schalten\r\n","");
					PORTC = PORT_tmp; // Ausgang C schalten	
                	PORT_tmp = 0;
				}
				else // nicht die Indexseite (setup.htm ....)
				{
					HTTP_DEBUG("Genderte Eintrge speichern\r\n","");
					save_ip_addresses();
				}
				break;
			}	// if(*http_entry[index].post_ready_ptr == 0)
		}	// for(a = TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
	}	// if(http_entry[index].http_auth && http_entry[index].post == 1)	
	
	//Welche Datei wird angefordert? Wird diese in der Flashspeichertabelle gefunden?
	unsigned char page_index = 0;
	
	if (!http_entry[index].new_page_pointer)
	{
		for(a = TCP_DATA_START_VAR+5;a<(TCP_DATA_END_VAR);a++)
		{
			if (eth_buffer[a] == '\r')
			{
				eth_buffer[a] = '\0';
				break;
			}
		}	
		while(WEBPAGE_TABLE[page_index].filename)
		{
			if (strcasestr((char*)&eth_buffer[TCP_DATA_START_VAR],WEBPAGE_TABLE[page_index].filename)!=0) 
				{
					http_entry[index].http_header_type = 1;
					HTTP_DEBUG("\r\n\r\nDatei gefunden: ");
					HTTP_DEBUG("%s",(char*)WEBPAGE_TABLE[page_index].filename);
					HTTP_DEBUG("<----------------\r\n\r\n");	
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".jpg")!=0)
					{
						#if USE_CAM
						if (strcasestr(WEBPAGE_TABLE[page_index].filename,"camera")!=0)
						{	
							http_entry[index].cam = 1;
						}
						#endif //USE_CAM
						http_entry[index].http_header_type = 1;
					}
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".gif")!=0)
					{
						http_entry[index].http_header_type = 1;
					}	
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".htm")!=0)
					{
						http_entry[index].http_header_type = 0;	
					}	
					http_entry[index].new_page_pointer = WEBPAGE_TABLE[page_index].page_pointer;
					break;
				} // if (strcasestr((char*)&eth_buffer[TCP_DATA_START_VAR],WEBPAGE_TABLE[page_index].filename)!=0) 
			page_index++;
		}	// while(WEBPAGE_TABLE[page_index].filename)
	}	// if (!http_entry[index].new_page_pointer)

	//Wurde das Ende vom Header nicht erreicht
	//kommen noch weitere Stcke vom Header!
	if ((*http_entry[index].hdr_end_pointer != 0) || (http_entry[index].post == 1))
	{
		//Der Empfang wird Quitiert und es wird auf weiteres Headerstck gewartet
		tcp_entry[index].status =  ACK_FLAG;
		create_new_tcp_packet(0,index);
		//Warten auf weitere Headerpackete
		tcp_entry[index].app_status = 1;
		return;
	}	
	
	//Wurde das Passwort in den ganzen Headerpacketen gefunden?
	//Wenn nicht dann ausfhren und Passwort anfordern!
	if((!http_entry[index].http_auth) && tcp_entry[index].status&PSH_FLAG)
	{	
		//HTTP_AUTH_Header senden!
		http_entry[index].new_page_pointer = Page0;
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header3,(sizeof(http_header3)-1));
		tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
		create_new_tcp_packet((sizeof(http_header3)-1),index);
		tcp_entry[index].app_status = 2;
		return;
	}
	
	//Standart INDEX.HTM Seite wenn keine andere gefunden wurde
	if (!http_entry[index].new_page_pointer)
	{
		//Besucher Counter
		var_array[MAX_VAR_ARRAY-1]++;
		
		http_entry[index].new_page_pointer = Page1;
		http_entry[index].http_header_type = 0;
	}	
	
	tcp_entry[index].app_status = 2;
	//Seiten Header wird gesendet
	if(http_entry[index].http_header_type == 1)
	{
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header2,(sizeof(http_header2)-1));
        tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
        create_new_tcp_packet((sizeof(http_header2)-1),index);
        return;
	}
     
	if(http_entry[index].http_header_type == 0)
	{
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header1,(sizeof(http_header1)-1));
        tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
        create_new_tcp_packet((sizeof(http_header1)-1),index);
        return;
	}
    return;
}

//----------------------------------------------------------------------------
//Daten Packete an Client schicken
void httpd_data_send (unsigned char index)
{	
	unsigned int a;
	unsigned char str_len;
	
	char var_conversion_buffer[CONVERSION_BUFFER_LEN];
	
	//Passwort wurde im Header nicht gefunden
	if(!http_entry[index].http_auth)
	{
		http_entry[index].new_page_pointer = Page0;
		#if USE_CAM
		http_entry[index].cam = 0;
		#endif //USE_CAM
	}
	
	#if USE_CAM //*****************************************************************
	unsigned long byte_counter = 0;
	
	if(http_entry[index].cam > 0)
	{
        //Neues Bild wird in den Speicher der Kamera geladen!
		if(http_entry[index].cam == 1){
			max_bytes = cam_picture_store(CAM_RESOLUTION);
			http_entry[index].cam = 2;
		}
        
		for (a = 0;a < (MTU_SIZE-(TCP_DATA_START)-10);a++)
		{
			byte_counter = ((tcp_entry[index].app_status - 3)*(MTU_SIZE-(TCP_DATA_START)-10)) + a;
			
			eth_buffer[TCP_DATA_START + a] = cam_data_get(byte_counter);
			
			if(byte_counter > max_bytes)
			{
				tcp_entry[index].app_status = 0xFFFD;
				a++;
				break;
			}
		}
		//Erzeugte Packet kann nun gesendet werden!
		tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
		create_new_tcp_packet(a,index);
		return;	
	}
	#endif //USE_CAM***************************************************************

	//kein Packet empfangen Retransmission des alten Packetes
	if (tcp_entry[index].status == 0) 
	{
		http_entry[index].new_page_pointer = http_entry[index].old_page_pointer;
	}
	http_entry[index].old_page_pointer = http_entry[index].new_page_pointer;

	for (a = 0;a<(MTU_SIZE-(TCP_DATA_START)-150);a++)
	{
		unsigned char b;
		b = pgm_read_byte(http_entry[index].new_page_pointer++); // Zeichen aus Buffer holen und Pointer um 1 erhhen
		eth_buffer[TCP_DATA_START + a] = b;
		
		//Mssen Variablen ins Packet eingesetzt werden? ===> %VA@00 bis %VA@09
		if (b == '%')
		{
			// Messwerte fr ADC-Ports bergeben
			if (strncasecmp_P("VA@",http_entry[index].new_page_pointer,3)==0) // "VA@" im Buffer ?
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; // b aus %VAxx generieren
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 // b aus %VAxx generieren
				itoa (var_array[b],var_conversion_buffer,10);				 	 // b in ASCII wandeln
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len); // B in Sendebuffer kopieren
				a = a + (str_len-1);					// Zhler der in den Buffer geschriebenen Zeichen
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5; // Zeiger auf Sourcefile
			}

			// Werte fr Balkenanzeige der ADC-Ports bergeben
			if (strncasecmp_P("VB@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa ((var_array[b] / 2),var_conversion_buffer,10);			// 50% des Messwerts = Balkenlnge			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// MAC-Adresse bergeben 
			if (strncasecmp_P("MC@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (mymac[b],var_conversion_buffer,16);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// IP-Adresse bergeben 
			if (strncasecmp_P("MP@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (myip[b],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// Netmask bergeben 
			if (strncasecmp_P("NM@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (netmask[b],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// Gateway-Adresse bergeben 
			if (strncasecmp_P("GW@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (router_ip[b],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// Deltawert der Ausgabe der Analogwerte bergeben 
			if (strncasecmp_P("DT@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa(delta[0],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// aktive CMD-Schnittstelle bergeben 
			if (strncasecmp_P("TN@",http_entry[index].new_page_pointer,3)==0)//
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (ctel,var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// Username bergeben 
			if (strncasecmp_P("UN@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (user[0],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			// Passwort bergeben 
			if (strncasecmp_P("PW@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10; 
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	 
				itoa (pass[0],var_conversion_buffer,10);			 
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);					
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

//            #if USE_NTP
            //Zeit in Webseite einfgen
//            if (strncasecmp_P("TIME",http_entry[index].new_page_pointer,4)==0)
//            {
//                if (ntp_state == NTP_STATE_REQ_ERR)
//                {
//                    strcpy("dstr","-");
//                }
//                else
//                {
//                    ntp_struct dt;
//                    decode_time(time, &dt);
//                    sprintf_P(dstr, PSTR("%s, %s"), dt.datestr, dt.timestr);
//                }
//                str_len = strnlen(dstr,24);
//                memmove(&eth_buffer[TCP_DATA_START+a],dstr,str_len);
//                a += str_len - 1;
//                http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+4;
//            }
//            #endif
            
			//Einsetzen des Port Status %PORTxy durch "checked" wenn Portx.Piny = 1
			//x: A..G  y: 0..7 
			if (strncasecmp_P("PORT",http_entry[index].new_page_pointer,4)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+5)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+4))
				{
					case 'A':
						b = (PORTA & (1<<pin));
						break;
					case 'B':
						b = (PORTB & (1<<pin));
						break;
					case 'C':
						b = (PORTC & (1<<pin));
						break;
					case 'D':
						b = (PORTD & (1<<pin));
						break; 
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR("checked"));
				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("\0"));
				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+6;
			}
			
			//Einsetzen des Pin Status %PI@xy bis %PI@xy durch "ledon" oder "ledoff"
			//x = 0 : PINA / x = 1 : PINB / x = 2 : PINC / x = 3 : PIND
			if (strncasecmp_P("PIN",http_entry[index].new_page_pointer,3)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+3))
				{
					case 'A':
						b = (PINA & (1<<pin));
						break;
					case 'B':
						b = (PINB & (1<<pin));
						break;
					case 'C':
						b = (PINC & (1<<pin));
						break;
					case 'D':
						b = (PIND & (1<<pin));
						break;    
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR("ledon.gif"));
				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("ledoff.gif"));
				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+5;
			}
			//wurde das Ende des Packetes erreicht?
			//Verbindung TCP Port kann beim nchsten ACK geschlossen werden
			//Schleife wird abgebrochen keine Daten mehr!!
			if (strncasecmp_P("END",http_entry[index].new_page_pointer,3)==0)
			{	
				tcp_entry[index].app_status = 0xFFFD;
				break;
			}
		}
	}
	//Erzeugte Packet kann nun gesendet werden!
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	create_new_tcp_packet(a,index);
	return;
}


