/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich  -  angepasst an AVR-NET-IO durch ** EHB **
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "networkcard/rtl8019.h"
#include "stack.h"
#include "timer.h"
#include "wol.h"
#include "httpd.h"
#include "cmd.h"
#include "telnetd.h"
#include "ntp.h"
#include "base64.h"
#include "http_get.h"
#include "lcd.h"
#include "udp_lcd.h"
#include "analog.h"
#include "camera/cam.h"
#include "camera/servo.h"
#include "sendmail.h"
#include "dhcpc.h"
#include "dnsc.h"

unsigned int first_run;	// Variable zur Kennzeichnung --> Modul neu gestartet
unsigned long keepalive_zeit; // Zeit des letzten Telnet-Packets

//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{  
	// Konfiguration der Ausgnge bzw. Eingnge
	// Definition erfolgt in der config.h
	DDRA = OUTA;	// Eingnge
	DDRC = OUTC;	// Ausgnge
	DDRD = OUTD;	// Eingnge

	first_run = 1; // Modul wurde neu gestartet
	keepalive_zeit = time; // aktuelle Zeit (UNIX-Format) bernehmen
    unsigned long a; //  Zhlvariable

    usart_init(BAUDRATE); // UART initialisieren
	
	#if USE_ADC
		ADC_Init();	// ADC initialisieren
	#endif
	
	usart_write("\n\rSystem Ready\n\r");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
    
	for(a=0;a<1000000;a++){asm("nop");}; 						// Pause ....

	// Applikationen starten
	stack_init(); // TCP/IP-Stack starten
	httpd_init(); // HTTP-Server starten
	telnetd_init(); // Telnet-Server starten
	
	// Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	// Globale Interrupts einschalten
	sei(); 
	
    #if USE_DHCP
    dhcp_init(); // DHCP-Client starten
    if ( dhcp() == 0)
    {
        save_ip_addresses();
    }
    else
    {
        usart_write("DHCP fail\r\n");
        read_ip_addresses(); //get from EEPROM - siehe "CMD.C"
    }
    #endif //USE_DHCP
	
	// Eingestellte Adressen / Werte anzeigen
	usart_write("\r\nMAC   %1x.%1x.%1x.%1x.%1x.%1x\r\n", mymac[0], mymac[1], mymac[2], mymac[3], mymac[4], mymac[5]);
    usart_write("IP   %1i.%1i.%1i.%1i\r\n", myip[0]     , myip[1]     , myip[2]     , myip[3]);
	usart_write("MASK  %1i.%1i.%1i.%1i\r\n", netmask[0]  , netmask[1]  , netmask[2]  , netmask[3]);
    usart_write("GW   %1i.%1i.%1i.%1i\r\n", router_ip[0], router_ip[1], router_ip[2], router_ip[3]);
	usart_write("Delta  %1i\r\n", delta[0]);
	usart_write("Telnet  %1i\r\n", ctel);

    #if USE_DNS
    usart_write("DNS  %1i.%1i.%1i.%1i\r\n", dns_server_ip[0], dns_server_ip[1], dns_server_ip[2], dns_server_ip[3]);
    #endif //USE_DNS
    
	#if USE_WOL
        wol_init();
	#endif //USE_WOL
    

	// Hauptschleife ***********************************************************************	
	while(1)
	{
		#if USE_ADC
		ANALOG_ON;
		#endif
	    eth_get_data();
		
        //Terminalcommandos auswerten
		if (usart_status.usart_ready)
		{	
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write(STX ACK ETX); // ACK-Meldung senden
				keepalive_zeit = time;
			}
			else
			{
				usart_write(STX NAK ETX); // NAK-Meldung senden
				keepalive_zeit = time;
			}
			usart_status.usart_ready = 0;
		}
		    
        //Rechner im Netzwerk aufwecken
        #if USE_WOL
        if (wol_enable == 1)
        {
            wol_enable = 0;
            wol_request();
        }
        #endif //USE_WOL
        
        #if USE_DHCP
        if ( dhcp() != 0) //check for lease timeout
        {
            usart_write("dhcp lease renewal failed\r\n");
			RESET();
        }
        #endif //USE_DHCP

		// Reset-Taste auf nderungen abfragen
		if((PIND&0b00010000) != 16)
		{
			usart_write("Modul-Reset durch Reset-Taste\r\n");
			RESET(); // Modul-Reset
		}
		// Init-Taste auf nderungen abfragen
		if((PIND&0b00100000) != 32)
		{
			usart_write("Grundeinstellungen wieder hergestellt durch Init-Taste\r\n");
		 	modul_init(); // Grundeinstellungen wieder herstellen
		}

		// Digitale Eingnge auf nderungen abfragen 
		command_getdinx(0);
		command_getdinx(1);
		command_getdinx(2);
		command_getdinx(3);
		// Analoge Eingnge auf nderungen abfragen 
		command_getainx(4);
		command_getainx(5);
		command_getainx(6);
		command_getainx(7);

		telnetd_send_data();
        
		// Keep-Alive Meldung alle 60 Sekunden
		if (keepalive_zeit == (time - 60))
		{
			usart_write(STX "00" ETX);  	// Meldung senden
			keepalive_zeit = time;		   // Zeit zurcksetzen
		} 

		if (Telnet_aktiv != 1) // Telnetverbindung nicht mehr aktiv ?
		{
			first_run = 1; // Modul wurde neu gestartet
		}

		// Startmeldung nach Neustart des Telnet-Servers
		if ((first_run == 1) && (Telnet_aktiv == 1))
		{
			for(a=0;a<1000000;a++){asm("nop");}	// Pause ...
			usart_write(STX "03" ETX); // Meldung senden
			keepalive_zeit = time; // aktuelle Zeit bernehmen
			first_run = 0; 
		} 
    }	//	End while (1)
return(0);
}
